;
;      SCRIPT: BRANCH2.SC
;      AUTHOR: Matthew Hopkins
; DESCRIPTION: This script contains PAL code to help explain the concept of
;              branching dynarrays.  More specifically, this script displays
;              a dialog box demonstrating a 1-M-M relationship between
;              dynarrays using this branching concept.
;

PROC CreateArrays() ; initialize variables

  DYNARRAY Grocery.y[]
  Grocery.y["Produce.y"] = "Produce"
  Grocery.y["Canned.y"]  = "Canned"

  DYNARRAY Produce.y[]
  Produce.y["Fruit.y"]      = "Fruit"
  Produce.y["Vegetables.y"] = "Vegetables"

  DYNARRAY Fruit.y[]
  Fruit.y["Apples.y"]  = "Apples"
  Fruit.y["Lemons.y"]  = "Lemons"
  Fruit.y["Oranges.y"] = "Oranges"

  DYNARRAY Vegetables.y[]
  Vegetables.y["Carrots.y"] = "Carrots"
  Vegetables.y["Celery.y"]  = "Celery"
  Vegetables.y["Lettuce.y"] = "Lettuce"

  DYNARRAY Canned.y[]
  Canned.y["Soup.y"]        = "Soup"
  Canned.y["CannedMeat.y"]  = "Canned Meat"

  DYNARRAY Soup.y[]
  Soup.y["Tomato.y"]            = "Tomato"
  Soup.y["CreamofMushroom.y"]   = "Cream of Mushroom"
  Soup.y["ChickenNoodle.y"]     = "Chicken Noodle"

  DYNARRAY CannedMeat.y[]
  CannedMeat.y["Tuna.y"]   = "Tuna"
  CannedMeat.y["Salmon.y"] = "Salmon"
  CannedMeat.y["SPAM.y"]   = "SPAM"

  DYNARRAY TopArray.y[]
  DYNARRAY MiddleArray.y[]
  DYNARRAY BottomArray.y[]
  TopArrayName.a    = "Grocery.y"
  MiddleArrayName.a = "Produce.y"
  BottomArrayName.a = "Fruit.y"

ENDPROC

PROC GenerateDisplayArray(ArrayName.a,currentarray.a)
  executestring.a = "DYNARRAY "+ArrayName.a+".y[] "+
                    "FOREACH element.a IN "+CurrentArray.a+" "+
                    "  "+ArrayName.a+".y[element.a] = "+CurrentArray.a+"[element.a] "+
                    "ENDFOREACH"
  IF (LEN(executestring.a)<=255) THEN
    EXECUTE executestring.a
  ELSE
    FILEWRITE "$BRANCH$.SC" FROM executestring.a
    PLAY "$BRANCH$"
  ENDIF
ENDPROC

PROC Dialog()
  SHOWDIALOG "Groceries"
    PROC "DialogControl"
    TRIGGER "SELECT"
    @3,22 HEIGHT 19 WIDTH 33

    PICKDYNARRAY @1,1 HEIGHT 3 WIDTH 27
      COLUMNS 1
      TopArray.y
      TAG "TopArray"
      TO TopArray.a

    PICKDYNARRAY @5,1 HEIGHT 4 WIDTH 27
      COLUMNS 1
      MiddleArray.y
      TAG "MiddleArray"
      TO MiddleArray.a

    PICKDYNARRAY @10,1 HEIGHT 4 WIDTH 27
      COLUMNS 1
      BottomArray.y
      TAG "BottomArray"
      TO BottomArray.a

    PUSHBUTTON @15,7 WIDTH 6
      "Ok"
      OK
      VALUE "OK"
      TAG "OKButton"
      TO ButtonValue.a

    PUSHBUTTON @15,13 WIDTH 10
      "Cancel"
      CANCEL
      VALUE "Cancel"
      TAG "CancelButton"
      TO ButtonValue.a
 ENDDIALOG
ENDPROC

PROC DialogControl(EventType,TagValue,EventValue,ElementValue)
  PRIVATE where.n
  SWITCH
    CASE (EventType = "SELECT") :
      SWITCH
        CASE (TagValue = "TopArray") :
          MiddleArrayName.a = CONTROLVALUE(TagValue)
          GenerateDisplayArray("MiddleArray",MiddleArrayName.a)
          REFRESHDIALOG
          BottomArrayName.a = MiddleArray.a
          GenerateDisplayArray("BottomArray",BottomArrayName.a)
          REFRESHDIALOG
        CASE (TagValue = "MiddleArray") :
          BottomArrayName.a = CONTROLVALUE(TagValue)
          RefreshArrays()
          REFRESHDIALOG
      ENDSWITCH
  ENDSWITCH
ENDPROC

PROC RefreshArrays()
  GenerateDisplayArray("TopArray",TopArrayName.a)
  GenerateDisplayArray("MiddleArray",MiddleArrayName.a)
  GenerateDisplayArray("BottomArray",BottomArrayName.a)
ENDPROC

;
CreateArrays()
RefreshArrays()
Dialog()
